import discord
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.logging.command_logger import CommandLogger
from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC


class AutoRoleCommand(DiscordCommandABC):

    def __init__(
            self,
            logger: CommandLogger,
            message_service: MessageServiceABC,
            bot: DiscordBotServiceABC,
            client_utils: ClientUtilsServiceABC,
            translate: TranslatePipe,
            servers: ServerRepositoryABC,
            auto_roles: AutoRoleRepositoryABC
    ):
        DiscordCommandABC.__init__(self)

        self._logger = logger
        self._message_service = message_service
        self._bot = bot
        self._client_utils = client_utils
        self._t = translate
        self._servers = servers
        self._auto_roles = auto_roles

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.hybrid_group(name="auto-role")
    @commands.guild_only()
    async def auto_role(self, ctx: Context):
        pass

    @auto_role.command()
    @commands.guild_only()
    async def list(self, ctx: Context, wait: int = None):
        self._logger.debug(__name__, f'Received command auto-role list {ctx}')
        if not await self._client_utils.check_if_bot_is_ready_yet_and_respond(ctx):
            return
        self._client_utils.received_command(ctx.guild.id)

        embed = discord.Embed(
            title=self._t.transform('modules.auto_role.list.title'),
            description=self._t.transform('modules.auto_role.list.description'),
            color=int('ef9d0d', 16)
        )

        auto_role_id = ''
        for id in self._auto_roles.get_auto_roles().select(lambda x: x.auto_role_id):
            auto_role_id += f'\n{id}'

        message_id = ''
        for id in self._auto_roles.get_auto_roles().select(lambda x: x.discord_message_id):
            message_id += f'\n{id}'
        embed.add_field(name=self._t.transform('modules.auto_role.list.auto_role_id'), value=auto_role_id, inline=True)
        embed.add_field(name=self._t.transform('modules.auto_role.list.message_id'), value=message_id, inline=True)
        await self._message_service.send_ctx_msg(ctx, embed, wait_before_delete=wait)
        self._logger.trace(__name__, f'Finished command auto-role list')

    @auto_role.command()
    @commands.guild_only()
    async def add(self, ctx: Context):
        self._logger.debug(__name__, f'Received command auto-role add {ctx}')
        if not await self._client_utils.check_if_bot_is_ready_yet_and_respond(ctx):
            return

    @auto_role.command()
    @commands.guild_only()
    async def remove(self, ctx: Context):
        self._logger.debug(__name__, f'Received command auto-role remove {ctx}')
        if not await self._client_utils.check_if_bot_is_ready_yet_and_respond(ctx):
            return
